#!/bin/bash
#
#
# pvr record [channel-alias] at [start-time] until [end-time] 
#             $2             $3  $4          $5     $6 
# pvr record [channel-alias] at [start-time] for [end-time] name customname margins 2-4
#             $2             $3  $4          $5   $6        $7/8            $9/10
#
# ------------------------------
#
# announce recording
# scheduling; explain schedule on change
# do not overwrite existing file
# can open dvb front end?
# switch off without hosing all mplayer processes!
# auto-repair config toggle
# auto transcode config toggle..?
# run user defined script, if present?
#
# ---------------------------------


sources=(
    '/etc/pvr/config.sh'
    '/opt/pvr/lib/time.sh'
    '/opt/pvr/lib/recorder.sh'
    '/opt/pvr/lib/recorder.sh'
)
for s in ${sources[@]}
do
	if [ -f $s ]
	then
        source $s
    else
		echo 'source file ['${s}'] not exists'
        exit 1
	fi
done

# ---------------------------------

have_custom_margins=0
now=`date +%s`

# gather ---------------------------------

channel="$2"
start_time="$4"

syntaxCheck_1 $@
syntaxCheck_2 $@

if [ "$5" = "until" ]
then
	end_time="$6"
fi
if [ "$5" = "for" ]
then
	duration="$6"	
fi

if [ $# -eq 8 ] && [ "$7" = "margins" ]
then
    have_custom_margins=1
    margins=$8
fi
if [ $# -eq 10 ] && [ "$9" = "margins" ]
then
    have_custom_margins=1
    margins=$10
fi

recording_name=`setRecordingName $@`


# check ---------------------------------------------------

if [ 'false' = `isValidChannel "$channel" "$channels_conf"` ]
then
    "channel [$channel] not found in [$channels_conf]"
	exit 1
fi

if [ 'unknown' = "`getTimeFormat $start_time`" ]
then
    echo
    echo 'start time has unknown or ambiguous format ['${start_time}']'
    echo 'accepted time formats: HH:mm (24hr) or H:mm[am/pm] or HH:mm[am/pm] (12hr)'
	echo
    recorderHelp
    exit 1
fi

if [ "$5" = "until" ] && [ 'unknown' = "`getTimeFormat $end_time`" ]
then
    echo
    echo 'end time has unknown or ambiguous format ['${end_time}']'
    echo 'accepted time formats: HH:mm (24hr) or H:mm[am/pm] or HH:mm[am/pm] (12hr)'
    echo
	recorderHelp
    exit 1
fi

if [ "$5" = "for" ] && [ 'unknown' = `getDuration $duration` ]
then
    echo
    echo 'duration has unknown or ambiguous format ['${duration}']'
    echo 'accepted formats: H:mm or X[h|m|s]'
	echo
    recorderHelp
    exit 1
fi

if [ 1 -eq $have_custom_margins ] && [ "" = "`echo $margins | grep ^[0-9]*-[0-9]*$`" ]
then
    echo
    echo 'margins has unknown or ambiguous format ['${margins}']'
    echo 'accepted format: "x-y" where x and y are integers defining '
    echo 'pre and post margins, respectively, in minutes'
    echo
  	recorderHelp
    exit 1	
fi

# ---------------------------------------------------

start_time=`c24hr "$start_time"`
recording_name=`appendDate "$recording_name" $start_time`
setDuration $5
setMargins
wait_for_start=`getWaitForStart $now $start_time $duration $pre_margin $post_margin`
wait_for_end=`getWaitForEnd $now $start_time $duration $pre_margin $post_margin`

if [ $wait_for_start -gt 0 ]
then
    echo 'Recording start time is '$start_time' minus safety margin '${pre_margin}'s '.
    sleep $wait_for_start
fi

echo 'start recording...'
mplayer -dumpstream "dvb://$channel" -dumpfile ${pvr_scratch_dir}${recording_name} &
pid=$!
sleep $wait_for_end
kill $pid
echo
echo 'recording saved to ['${pvr_scratch_dir}${recording_name}']'
echo
exit 0
