#!/bin/bash

#
# repair a dvb recording (.ts file) 
#

# pvr_scratch_dir is the dir where the dvb recording has been saved
# if possible, $pvr_scratch_dir and $video_tmpdir should be on two 
# different HDDs

#-------------------

if [ -f $config ]
then
    source $config
else
	echo "cannot source file [$config]"
    exit 1
fi
if [ ! -d $pvr_scratch_dir ]
then
    echo "scratch dir [$pvr_scratch_dir] not exists"
    exit 1
fi
if [ ! -d $video_tmpdir ]
then
    echo "video tmp dir [$video_tmpdir] not exists"
    exit 1
fi
if [ "$1" ]
then
    tspath="$1"
else
    showHelp 'repair'    
    exit 1
fi
if [  ! -f "$tspath"  ]
then
    echo "$tspath is not a file"
    exit 1
fi

if [ ! `echo "$tspath" | grep [.]ts$` ]
then
    echo "$tspath is not a .ts file"
    exit 1
fi


#-------------------

title=`basename $tspath | sed s/[.][a-zA-Z0-9]*$//`
repaired=${pvr_scratch_dir}${title}'_repaired.mpeg2' 
video=${video_tmpdir}${title}'.m2v' 
audio=${video_tmpdir}${title}'.mp2'
log=${video_tmpdir}${title}'_log.txt'


#echo 'title: '$title
#echo 'repaired: '$repaired
#echo 'log: '$log
#echo 'video: '$video
#echo 'audio: '$audio

if [ "`echo $title | grep [.]`" ]
then
    echo "I don't think projectx can handle [.] in the parsed \$title [$title]."
    echo 'exiting...'
    exit 1
fi

# http://www.doom9.org/index.html?/DigiTV/projectx-fullguide.htm
# "In digital TV, the picture is compressed using the well-known MPEG-2 
# format, before being broadcast as digital data stream (MPEG2 transport 
# stream - MPEG-2 TS). MPEG-2 transport streams are part of the MPEG-2 
# standard, but most tools cannot directly handle transport streams as 
# its structure is different from MPEG-2 elementary streams we use on 
# DVDs or SVCDs. This is where ProjectX enters the picture. It works 
# like an MPEG-2 TS hardware decoder, splitting the transport stream into 
# its components (MPEG elementary streams like audio, video, and other data 
# (e.g. teletext)). While splitting the transport stream, ProjectX also checks 
# the stream for errors and synchronizes video and audio. If ProjectX converts 
# the transport stream to another format, it won't analyze and fix streams as 
# thoroughly as in split mode.
projectx -demux $tspath -out $video_tmpdir -name $title

if [ ! -f $video ]
then
  echo 'video ['$video'] does not exist'
  exit 1
fi
if [ ! -f $audio ]
then
  echo 'video ['$audio'] does not exist'
  exit 1
fi
        
mplex -f 8 -o $repaired $video $audio

if [ ! -f $repaired ]
then
  echo 'repaired ['$repaired'] does not exist'
  exit 1
fi

echo 
echo -n "All done. The repaired file is: [$repaired]"

echo
echo 'deleting the following files:'
echo $tspath
echo $video
echo $audio
echo $log

#read ANSWER
#if [ "$ANSWER" == 'y' ]; then
    rm $tspath
    rm $video
    rm $audio
    rm $log
#fi
exit 0
