<?php

class DvbtChannels {
    
    function __construct(
        $reporter, 
        $persistence, 
        $newscan_path, 
        $channels_conf_path) {
        
        $this->_reporter = $reporter;
        $this->_persistence = $persistence;
        $this->_newscan_path = $newscan_path;
        $this->_channels_conf_path = $channels_conf_path;
    }
    function update() {
        $this->_newly_scanned_channels = new ChannelsConfReader($this->_newscan_path);
        while(false !== ($newly_scanned = $this->_newly_scanned_channels->next())) {
            if($this->_hasNoPids($newly_scanned)) {
                $this->_reporter->noPidsFor($newly_scanned[0]);
                continue;
            }
            if($this->_isNewNameOnUsedSlot($newly_scanned)) {
                # reported below
                continue;
            }
            if($this->_isNewChannel($newly_scanned)) {
                $this->_persistence->addChannel($newly_scanned);
                $this->_reporter->addingChannel($newly_scanned[0]);
                continue;
            }
            if($this->_persistence->isChanged($newly_scanned)) {
                $this->_persistence->updateChannel($newly_scanned);
                $this->_reporter->updatingChannel($newly_scanned[0]);
                continue;
            }
        }
        $this->_writeChannelsConf();
        $this->_reporter->done();
    }
    function _hasNoPids($channel) {
        return ($channel[10] == 0 and $channel[11] == 0);
    }
    function _isNewNameOnUsedSlot($channel) {
        if($this->_persistence->findChannelByName($channel[0])) {
            return false;
        }
        $new_name = array_shift($channel);
        $slot_data = $channel;
        $rows = call_user_func_array(
            array($this->_persistence, 'findChannelsOnSlot'), 
            $slot_data);
        if($rows->numRows() > 0) {
            $row = $rows->next();
            $this->_reporter->newNameOnSlot($new_name, array($row['custom_name'], $row['name']));
            return true;
        } else {
            return false;
        }
    }
    function _isNewChannel($channel) {
        return (bool)!$this->_persistence->findChannelByName($channel[0]);
    }
    function _writeChannelsConf() {
        $handle = fopen($this->_channels_conf_path, 'wb');
        if( !$handle ) {
            $this->_reporter->error(
                "cannot open channels.conf [$this->_channels_conf_path] for writing");
            return false;
        }
        foreach($this->_persistence->getChannelsUsingCustomNames() as $channel) {
            fwrite($handle, implode(':', $channel) . "\n");
        }
        fclose($handle);
        return true;
    }
    function renameChannel($scanned_name, $custom_name) {
        $affected = $this->_persistence->renameChannel($scanned_name, $custom_name);
        if(1 === $affected) {
            $this->_writeChannelsConf();
            $this->_reporter->renameChannel($scanned_name, $custom_name);
        } else {
            $this->_reporter->failedToRenameChannel($scanned_name, $custom_name, $affected);            
        }
    }
}

/*
*/
class MissingChannelsConf extends Exception {}
class UnreadableChannelsConf extends Exception {}
class EmptyChannelsConf extends Exception {}

class DvbtChannelsData {

    var $_add_multiplex = "insert into multiplexes (
                multiplex_frequency,
                inversion, 
                bandwidth, 
                fec1, 
                fec2, 
                qam, 
                transmission_mode, 
                guard_interval, 
                hierarchy
            ) values (
                %d, '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'
            )";
    var $_add_slot = "insert into slots (
                muid, 
                video_pid, 
                audio_pid, 
                unknown_id
            ) values (
                %d, %d, %d, %d
            )";
    var $_add_channel = "insert into channels (
                custom_name, 
                name, 
                sort_order, 
                slid
            ) values (
                '%s', '%s', %d, %d
            )";

    var $_select_channel = "select 
            c.name, 
            m.multiplex_frequency, 
            m.inversion, 
            m.bandwidth, 
            m.fec1, 
            m.fec2, 
            m.qam, 
            m.transmission_mode, 
            m.guard_interval, 
            m.hierarchy, 
            s.video_pid, 
            s.audio_pid, 
            s.unknown_id
        from channels as c
        inner join slots as s using (slid)
        inner join multiplexes as m using (muid)\n";
        
    var $_get_multiplex_id = "select muid from multiplexes where
            multiplex_frequency='%s' and 
            inversion='%s' and 
            bandwidth='%s' and
            fec1='%s' and
            fec2='%s' and
            qam='%s' and
            transmission_mode='%s' and 
            guard_interval='%s' and
            hierarchy='%s'";

    var $_get_slot_id = "select slid from slots where
            muid='%s' and
            video_pid='%s' and 
            audio_pid='%s' and
            unknown_id='%s'";

    var $_select_channel_2 = "select 
            c.chid, 
            c.slid, 
            m.muid
        from channels as c
        inner join slots as s using (slid)
        inner join multiplexes as m using (muid)
        where c.name='%s'";
    
    var $_select_channels_using_custom_names = "select 
            c.custom_name, 
            m.multiplex_frequency, 
            m.inversion, 
            m.bandwidth, 
            m.fec1, 
            m.fec2, 
            m.qam, 
            m.transmission_mode, 
            m.guard_interval, 
            m.hierarchy, 
            s.video_pid, 
            s.audio_pid, 
            s.unknown_id
        from channels as c
        inner join slots as s using (slid)
        inner join multiplexes as m using (muid)
        order by sort_order";

    var $_rename_channel = "update channels set custom_name='%s' where name='%s'";
    var $_select_custom_name = "select custom_name from channels where name='%s'";

    var $_delete_channel = "delete from channels where chid=%d";
    var $_delete_slot = "delete from slots where slid=%d";
    var $_delete_multiplex = "delete from multiplexes where muid=%d";
    var $_find_channels_by_slot_id = "select chid from channels where slid=%d";
    var $_find_slots_by_multiplex_id = "select slid from slots where muid=%d";

    var $_channels_on_slot = "select c.custom_name, c.name from channels as c 
                inner join slots as s using (slid)
                inner join multiplexes as m using (muid)
                where m.multiplex_frequency='%s'
                    and m.inversion='%s' 
                    and m.bandwidth='%s' 
                    and m.fec1='%s'
                    and m.fec2='%s'
                    and m.qam='%s'
                    and m.transmission_mode='%s'
                    and m.guard_interval='%s'
                    and m.hierarchy='%s'
                    and s.video_pid='%s'
                    and s.audio_pid='%s'
                    and s.unknown_id='%s'";

    function __construct($conn) {
        $this->_conn = $conn;
    }
    function addChannel($channel) {
        $value = $this->_getMultiplexId(
                        $channel[1], 
                        $channel[2], 
                        $channel[3], 
                        $channel[4], 
                        $channel[5], 
                        $channel[6], 
                        $channel[7], 
                        $channel[8], 
                        $channel[9]);

        if (false === $value) {
            $this->_conn->editTable(
                    $this->_add_multiplex, 
                    $channel[1], 
                    $channel[2], 
                    $channel[3], 
                    $channel[4], 
                    $channel[5], 
                    $channel[6], 
                    $channel[7], 
                    $channel[8], 
                    $channel[9]);
                
            $multiplex_id = $this->_conn->lastInsertId();

        } else {
            $multiplex_id = $value;
        }
        
        $value = $this->_getSlotId(
                $multiplex_id,  
                $channel[10], 
                $channel[11], 
                $channel[12]);

        if (false === $value) {
            $this->_conn->editTable(
                    $this->_add_slot, 
                    $multiplex_id,  
                    $channel[10], 
                    $channel[11], 
                    $channel[12]);        

            $slot_id = $this->_conn->lastInsertId();

        } else {
            $slot_id = $value;
        }

        $this->_conn->editTable(
                $this->_add_channel, 
                $channel[0], 
                $channel[0], 
                0, 
                $slot_id);
    }
    function _getMultiplexId(
        $multiplex_frequency,
        $inversion, 
        $bandwidth, 
        $fec1, 
        $fec2, 
        $qam, 
        $transmission_mode, 
        $guard_interval, 
        $hierarchy) {
                
        $row = $this->_conn->getRow(
            $this->_get_multiplex_id, 
            $multiplex_frequency,
            $inversion, 
            $bandwidth, 
            $fec1, 
            $fec2, 
            $qam, 
            $transmission_mode, 
            $guard_interval, 
            $hierarchy);

        if(is_array($row) and count($row)) {
            return $row['muid'];
        } else { 
            return false;
        }
    }
    function _getSlotId(
        $muid, 
        $video_pid, 
        $audio_pid, 
        $unknown_id) {
        
        $row = $this->_conn->getRow(
            $this->_get_slot_id, 
            $muid, 
            $video_pid, 
            $audio_pid, 
            $unknown_id);
            
        if(is_array($row) and count($row)) {
            return $row['slid'];
        } else { 
            return false;
        }
    }
    function findChannelByName($name) {
        $row = $this->_conn->getRow(
                $this->_select_channel . " where name='%s'", 
                $name);
        if(is_array($row)) {
            return array_values($row);
        } else {
            return $row;            
        }
    }
    function getChannels() {
        $array = array();
        $channels = $this->_conn->getRows(
                $this->_select_channel . " order by c.sort_order, c.name");
        while(false !== ($row = $channels->next())) {
            $array[] = array_values($row);
        }
        return $array;
    }
    function renameChannel($scanned_name, $custom_name) {
        $affected = $this->_conn->editTable(sprintf(
            $this->_rename_channel, 
            $custom_name, 
            $scanned_name));
        if($this->_conn->getError()) {
            trigger_error($this->_conn->getError());
        }
        return $affected;
    }
    function getChannelsUsingCustomNames() {
        $array = array();
        $channels = $this->_conn->getRows(
            $this->_select_channels_using_custom_names);
        while(false !== ($row = $channels->next())) {
            $array[] = array_values($row);
        }
        return $array;
    }
    function getChannelsWithNoPids() {
        $array = array();
        $channels = $this->_conn->getRows(
                $this->_select_channel . " where s.video_pid=0 and s.audio_pid=0");
        while(false !== ($row = $channels->next())) {
            $array[] = array_values($row);
        }
        return $array;
    }
    function isChanged($channel) {
        $stored = array_values($this->_conn->getRow(
            $this->_select_channel . 
            " where name='" . $channel[0] . "'"));
        return ($stored and $stored != $channel);
    }
    function deleteChannelByName($name) {
        $found = $this->_conn->getRow(
            $this->_select_channel_2, 
            $name);
        if($this->_conn->getError()) {
            trigger_error($this->_conn->getError());
        }
        $this->_conn->editTable(sprintf(
            $this->_delete_channel, 
            $found['chid']));
    }
    function updateChannel($channel) {
        $custom_name = $this->_getCustomName($channel[0]);
        $this->deleteChannelByName($channel[0]);
        $this->addChannel($channel);
        $this->renameChannel($channel[0], $custom_name);
    }
    function _getCustomName($scanned_name) {
        $row = $this->_conn->getRow(sprintf(
            $this->_select_custom_name, 
            $scanned_name));
        if($this->_conn->getError()) {
            trigger_error($this->_conn->getError());
        }
        return $row['custom_name'];
    }
    function findChannelsOnSlot(
        $multiplex, 
        $inversion, 
        $bandwidth, 
        $fec1, 
        $fec2, 
        $qam, 
        $transmission_mode, 
        $guard_interval, 
        $hierarchy, 
        $video_pid, 
        $audio_pid, 
        $unknown_id) {

        return $this->_conn->getRows(
            $this->_channels_on_slot, 
            $multiplex, 
            $inversion, 
            $bandwidth, 
            $fec1, 
            $fec2, 
            $qam, 
            $transmission_mode, 
            $guard_interval, 
            $hierarchy, 
            $video_pid, 
            $audio_pid, 
            $unknown_id);
    }
}
class ChannelsConfReader {

    var $_channels = array();
    
    function __construct($path) {
    
    echo "\n";
    echo "path:\n";
    echo `cat $path`;
    echo "\n";
    echo "\n";
    
    
        if( !file_exists($path)) {
            throw(new MissingChannelsConf);
        }
        if( !is_readable($path)) {
            throw(new UnreadableChannelsConf);
        }
        $this->_channels = array();
        foreach(file($path) as $channel) {
            if(trim($channel)) {
                $this->_channels[] = explode(':', trim($channel));
            }
        }
        if( !count($this->_channels)) {
            throw(new EmptyChannelsConf);
        }
    }
    function next() {
        return (!is_null($row = array_shift($this->_channels)) ? $row : false);
    }
    function numChannels() {
        return count($this->_channels);
    }
}

class DvbChannelsUpdateReporter {

    function addingChannel($name) {
        fwrite(STDOUT, "adding channel [$name]\n");
    }
    function updatingChannel($name) {
        fwrite(STDOUT, "updating channel [$name]\n");
    }
    function noPidsFor($name) {
        fwrite(STDOUT, "no pids in this scan for channel [$name]\n");
    }
    function foundPidsFor($name, $vpid, $apid) {
        fwrite(STDOUT, "found pids [$vpid, $apid] for [$name]\n");
    }
    function done() {
        fwrite(STDOUT, "\nmongo done now mongo done good\n\n");
    }
    function error($message) {
        fwrite(STDERR, "ERROR: $message\n\n");
    }
    function renameChannel($scanned_name, $custom_name) {
        fwrite(STDOUT, "renaming [$scanned_name] to [$custom_name]\n");
    }
    function failedToRenameChannel($scanned_name, $custom_name, $affected) {
        fwrite(STDOUT, "failed to rename [$scanned_name] to [$custom_name] affected [$affected]\n");
    }
    function newNameOnSlot($new, $old) {
        fwrite(STDOUT, "found new channel name [$new] on slot used by [$old[0]/$old[1]]\n");
    }
}

?>
