#
# Discover format of given time ie 22:00 (24hr), 1:45am or 10:00pm (12hr)
# 10:00 is always 10:00am in 24hr time
#
# $1 - time
#
getTimeFormat() {
    time=$1

    if [ "" !=  "`echo $1 | grep  ^[0-1][0-2]:[0-5][0-9][ap]m$`" ]
    then
        echo '12hr'
        return 0
    fi
    if [ "" !=  "`echo $1 | grep  ^[0-9]:[0-5][0-9][ap]m$`" ]
    then
        echo '12hr'
        return 0
    fi
    if [ "" !=  "`echo $1 | grep  ^[0-2][0-9]:[0-5][0-9]$`" ]
    then
        echo '24hr'
        return 0
    fi

    echo 'unknown'
    return 1
}

#
# $1 - time: H:mm or HH:mm
# $2 - optional +/- offset in secs
#
getSecondsFromNowTo() {
    local offset

    if [ $# -eq 1 ]
    then
    	offset=0
    else
        offset=$2
    fi
    echo "scale=0; $offset + `date +%s --date=$1` - `date +%s`" | bc -l
}
#
# $1 - "now" in unix seconds
# $2 - time: H:mm or HH:mm
# $3 - optional +/- offset in secs
#
getSecondsFromTo() {
    local offset

    if [ $# -lt 3 ]
    then
    	offset=0
    else
        offset=$3
    fi
    echo "scale=0; $offset + `date +%s --date=$2` - $1" | bc -l
}
#
# canonicalise input as 24hr time-of-day
#
# $1 - 22:00 (24hr), 1:45am or 10:00pm (12hr)
#
c24hr() {

    local hours=`echo "$1" | grep -o ^[^:]*`
    local mins=`echo "$1" | grep -o [^:]*$ | grep -o ^[^apm]*`
    local meridiem=`echo "$1" | grep -o [ap]m$`

    hours=$(($hours % 24))
    hours=$( printf %02d $hours )
    
    if [ 'am' = "$meridiem" ] && [ 12 -eq $hours ]
    then
        hours='00'
    fi
    if [ 'pm' = "$meridiem" ] && [ 12 -ne $hours ]
    then
        hours=$( echo "scale=0; 12+$hours" | bc -l )
    fi
    echo ${hours}':'${mins}
    return
}
#
# Return time in seconds from one of several possible time-duration 
# formats or return "unknown" if value not in a known format.
#
# $1 - time-duration in format: HH:MM, or Xh or Ym or Zs
#
getDuration() {
    local h
    local m

    if [ "" != "`echo $1 | grep ^[0-9]*s$`" ]
    then
    	echo $( echo $1 | sed 's/s//g' )
        return
    fi
    if [ "" != "`echo $1 | grep ^[0-9]*m$`" ]
    then
    	m=$( echo $1 | sed 's/m//g' )
        echo $(( m*60 ))
        return
    fi
    if [ "" != "`echo $1 | grep ^[0-9]*h$`" ]
    then
    	m=$( echo $1 | sed 's/h//g' )
        echo $(( m*60*60 ))
        return
    fi
    if [ "" != "`echo $1 | grep ^[0-9]*:[0-9][0-9]$`" ]
    then
        h=`echo "$1" | grep -o ^[^:]*`
        m=`echo "$1" | grep -o [^:]*$`
        h=$( printf %01d $h )
        echo $(( $h*60*60 + $m*60 ))
        return
    fi    
    
    echo 'unknown'
}
#
# Return (cyclical) time in seconds between two time-of-days.
#
# $1 - time-of-day (24hr)
# $2 - time-of-day (24hr)
#
getPeriod() {
    local first=`date +%s --date="$1"`
    local second=`date +%s --date="$2"`

    if [ $first -gt $second ]
    then
    	second=$(( $second + 24*60*60 ))
    fi
    echo $(( $second - $first ))
}
